#!/usr/bin/perl
use strict;
BEGIN {				# manual version of "use" for OS-dependency
    if ($^O =~ /win/i) {
	require File::DosGlob;
	import File::DosGlob;
    }
}


unless (@ARGV) {&usage and exit 1};

my $file;
my ($kprobe, $cdspeed) = (0, 0);
my %summary = ();
my @files = ();
foreach $file (@ARGV) {
    push @files, glob $file;
}
foreach $file (sort @files) {
    my $type = readlog($file);
    if ($type == 1) {$kprobe++};
    if ($type == 2) {$cdspeed++};
}
print STDERR "$kprobe KProbe file(s)\n";
print STDERR "$cdspeed CDSpeed file(s)\n";
print_kprobe() if ($kprobe);
print_cdspeed() if ($cdspeed);


sub usage {
print <<'ENDUSAGE';

convert information in KProbe2 or CDSpeed txt file to CSV list for Excel import

Usage: kptxt.pl <files> > txtsum.csv
ENDUSAGE
return 1;
}

#------------------------------------------------------------------------------
#		analyze KProbe log, add entry to summary hash
sub readlog {
    my $in = shift;

    open (IN, $in)
	or die "could not open $in for input: $!";

    my $line;
    my %scan = ();
    my $prefix = '';

    while ($line = <IN>) {
	chomp $line;
	my ($key, $val) = split (/ *\: +/, $line, 2);
	$val =~ s/\s+$//;
	if ($val ne "") {
	    $key =~ s(PI/PO)(PI/PIF);
	    $prefix = '' unless ($key =~ s/^ +//);	# CDSpeed subcategory
	    $scan{$prefix.$key} = $val;
	}
	elsif (%scan) {					# check CDSpeed subcategory
	    $prefix = '';
	    $prefix = $key.' ' if ($key =~ /^P/i or $key =~ /jitter/i);
	}
    }
    close IN
	or die "could not close $in: $!";

    my $type = 0;
    if (exists $scan{'Model'}) {
	$type = 1;				# KProbe2
	$scan{'KProbe version'} = '??' unless exists $scan{'KProbe version'};
    }
    elsif (exists $scan{'Firmware'}) {
	$type = 2;				# CDSpeed
	if (exists $scan{'Elapsed time'}) {
	    $scan{'Elapsed time'} = '0:'.$scan{'Elapsed time'};		# For excel time format
	}
	else {$scan{'Elapsed time'} = '??'};
#						add date from time stamp of file
	my $date = -M  $in;
	$scan{'Date'} = mytime($^T-86400*$date);
    }
    $summary{$in} = \%scan if ($type);
    return $type;
}
#------------------------------------------------------------------------------
#		analyze KProbe summary hash, print results to CSV file
sub print_kprobe {
#					specify keys in the proper order
    my @keys = (
	'KProbe version', 		# I add the version manually :D
	'Date', 
	'Model', 
	'Disc', 
	'Speed', 
	'ECC blocks sum (PI/PIF)', 
	'Scanned range', 
	'Sampling count', 
	'Errors', 
	'PI Max', 
	'PI Average', 
	'PI Total', 
	'PIF Max', 
	'PIF Average', 
	'PIF Total', 
    );
    print join(";", 'file', @keys), "\n";
    foreach my $file (sort keys %summary) {
	my $scan = $summary{$file};
	print join(";", $file, 
			    # quote some values that Excel thinks are dates
	    map {
		($_ =~ /ECC/ or $_ =~ /version/) ? "'" . $scan->{$_} : $scan->{$_}
	    } @keys), "\n";
    }
}

#------------------------------------------------------------------------------
#		analyze CDSpeed summary hash, print results to CSV file
sub print_cdspeed {
#					specify keys in the proper order
    my @keys = (
	'Date', 
	'Drive', 
	'Firmware', 
	'Disc', 
	'Selected speed', 
	'PI errors Maximum', 
	'PI errors Average', 
	'PI errors Total', 
	'PI failures Maximum', 
	'PI failures Average', 
	'PI failures Total', 
	'PO failures', 
	'Jitter', 
	'Jitter Maximum', 
	'Jitter Average', 
	'Elapsed time', 
	'Number of samples', 
	'Average scanning interval', 
	'Glitches removed', 
    );
#					find out which of these are present
    my %usedkeys = ();
    foreach my $file (sort keys %summary) {
	map {$usedkeys{$_}++} keys %{$summary{$file}};
    }
    my @usedkeys = grep {exists $usedkeys{$_}} @keys;

    print join(";", 'file', @usedkeys), "\n";
    foreach my $file (sort keys %summary) {
	my $scan = $summary{$file};
	print join(";", $file, map {exists $scan->{$_} ? $scan->{$_} : ''} @usedkeys), "\n";
    }
}
#-----------------------------------------------------------------------
#			format date/time as dd.mmm.yyyy hh:mm:ss
sub mytime {
    my $time = shift;
    my @months = qw(Jan Feb Mar Apr May Jun Jul Aug Sep Oct Nov Dec);
    my @time = localtime $time;

    $time = sprintf("%2.2d %s %4d %2.2d:%2.2d:%2.2d",
	    $time[3], $months[$time[4]], $time[5]+1900,
	    $time[2], $time[1], $time[0]);

    return $time;
}
